{% extends "../../../layouts/post.html" %}

{% block article %}

## Preface
I'm an indie game dev that has an interest in making tools that I think would improve my life as a game developer in the hopes that it can help others. I'm not an expert in any of the topics that this article covers. I have no formal education in game engine development or architecture, systems programming, or anything else. I went to university for Computer Science with a specialization in game design & development when I was in my early 20s and the only thing I learned is that there are better ways to spend tens of thousands of dollars.

Everything in this article is drawn directly from my experiences working in game development.

## Theseus' Shield
In the context of game design and development, object oriented data modeling can feel intuitive. Players, NPCs, and Enemies can all derive from a common "Creature" class which handle things like health and damage. Weapons, armor, and consumables can derive from a common Item class which handle inventory management.

Thinking exclusively in terms of hierarchal, nested patterns can come at a cost: software brittleness and inflexible design. Let's use an `Item` class as an example.

```cs
abstract class Item {
    public string Name { get; protected set; }
}

abstract class Weapon: Item {
    public float Damage { get; protected set; }
    public float Range { get; protected set; }
}

abstract class Armor: Item {
    public float Defense { get; protected set; }
}

class Sword: Weapon {}
class Bow: Weapon {}

class Helmet: Armor {}
class Shield: Armor {}
```

In the above scenario, we have [three layers of inheritance](https://wiki.c2.com/?MaxThreeLayersOfInheritance). We can imagine that `Item` handles behaviors like item management such as being added or removed from an inventory and highly generic item behavior. `Weapon` would then be responsible for an item which is capable of dealing damage at any range. We can expect that the specifics of its attack behavior would be implemented by a subclass. Similarly for `Armor`, this would handle generic damage interactions such as common mitigation or avoidance calculations while leaving specific implementation details to its subclasses.

Seasoned game devs, disciplined object-oriented programmers, and existing ECS developers can likely already see what comes next: a new requirement.

How do we add a Spiked Shield item -- one which derives the behaviors of `Weapon` and `Armor`? If we were using a language which supports multiple inheritance this could potentially be a nonissue: except that multiple inheritance is often [purposefully missing](https://en.wikipedia.org/wiki/Multiple_inheritance#The_diamond_problem) in many languages. We could decide that the item belongs more to one class than the other and just duplicate the missing behavior but these types of decisions often introduce unforeseen complexity: will the damage algorithm need to do a specific type check for `SpikedShield`? What about the equipment screen? And of course, what happens when we need to implement a damaging potion?

Perhaps the most appropriate solution for this case would be to forgo the inheritance pattern in favor of [composition](https://en.wikipedia.org/wiki/Composition_over_inheritance). In C#, this could be achieved with interfaces and default implementations.

```cs
interface ICollectable {
    void Take();
    void Remove();
}

interface IDamaging {
    // by default, we'll just pass the damage directly to
    // the receiving damage handler
    void ApplyDamage(IDamageable damageable, float value) {
        damageable.ReceiveDamage(this, value);
    }
}

interface IDamageable {
    void ReceiveDamage(IDamaging source, float value);
}

class SpikedShield: ICollectable, IDamaging, IDamageable {}
```

We can use this approach to refactor our existing items and systems to derive/override only the behaviors they use. The respective systems for these interfaces now only need to check for the existence of these interfaces in order to act on them.

Engines like Unity and Godot use variations of the [Entity-Component (EC)](https://gameprogrammingpatterns.com/component.html) pattern (similar to but distinct from Entity-Component-System (ECS)). These patterns favor composition over inheritance by allowing developers to isolate behaviors into discrete components that can be applied to entities. In the Spiked Shield example, a developer could make a "Damage Source" and "Damage Target" component and add both to the item. In essence, this is the same as the interface-based approach.

Unfortunately, these patterns dont alleviate other issues that are much more difficult to solve. 

## In the event of my demise
Due to the nature of video games, important events may need to be handled at any moment.  For example, a player may've dealt a fatal blow to a boss enemy on the same frame that they received fatal damage. In which order should this damage be processed? Depending on the handling order, this is likely the difference between clearing a potentially difficult boss battle and needing to do it again. 

In my experience, this would likely be handled by a traditional event system where the order is difficult to predict. This isn't to make the claim that syncronous event systems are *unpredictable*, but ensuring that a given event will be handled in a way that is predictable to the designer/developer without additional abstractions is difficult.

It's often useful to explicitly define the processing order of certain interactions and events. To solve this problem, we could implement a priority [event queue](https://gameprogrammingpatterns.com/event-queue.html).

```cs
public class DamageEventArgs : EventArgs {
    public readonly IDamaging Source;
    public readonly IDamageable Target;
    public readonly float Value;
}

// this is more of a dispatcher, but that's
// an unnecessary implementation detail.
// in a real scenario where other systems would be reading
// these events too, this would subscribe to a Mediator object
class DamageEvent {
    // only one of these should exist and should be globally
    // accessible so we make it a singleton
    private static DamageEvent instance = new Instance();
    public static DamageEvent Instance => instance;

    private PriorityQueue<DamageEventArgs, int> queue = new();

    private void Raise(DamageEventArgs args) {
        // before enqueuing, determine the priority
        // based on the object dealing damage
        var priority = args.Source switch {
            Player => 2,
            Enemy => 1,
            _ => 0
        };

        queue.Enqueue(args, priority);
    }

    // we'll dispatch events with the tick rate
    // so they're all handled at the same time
    private void OnTick(float deltaTime) {
        // for simplicity we'll dequeue everything each frame
        // and pass it to the damage handler
        while(queue.TryDequeue(out var e, out int priority)) {
            e.Source.ApplyDamage(e.Target, e.Value);
        }
    }
}

// bossEnemy's damage should be processed after player's damage
// if they're raised during the same frame
DamageEvent.Raise(new DamageEventArgs(bossEnemy, player, 10));
DamageEvent.Raise(new DamageEventArgs(player, bossEnemy, 10));

```

There's a lot of issues with this code that I'm going to pretend were deliberate decisions for brevity. The point of this example is to show that we can ingest events and sort them arbitrarily based on the requirements of the game. We've made a step in the right direction by identifying a need to explicitly order these events. Even if this implementation isn't ideal, it properly encodes the requirements of the design (ie. player damage should be processed before all other types).

There is still a timing issue with this approach however. Events can be raised at any point: before, during, or after `DamageEvent` queue has already done its work for the frame. If `bossEnemy` raises its damage event before `DamageEvent` processes its queue but `player` raises their event after, we still have the original issue.

Depending on the engine and implementation, there may be a few options for solving this. Rather than using `OnTick`, the damage can be handled in `OnLateTick` which runs after all `OnTick` systems have been processed (Unity's version of these methods are `Update` and `LateUpdate`). In Unity the `DamageEvent` singleton script could have its order explicitly modified in the settings or with the `DefaultExecutionOrder` attribute. In Godot, this would likely be resolved by moving the `DamageEvent` node lower in the tree since nodes are processed from top to bottom while resolving children first.

An alternative solution would be to identify the behavior which raises events and isolate it into its own singleton. Doing this would allow it to easily be ordered before the damage handling system. We'll revisit this idea later.

## Legion-oriented programming
Managing a game state as it grows in size and complexity is *difficult*. It's not uncommon for a game to have dozens, hundreds, or thousands of active entities at a time. In many cases, it makes sense to decouple separate-but-related behaviors into their own systems to make them easier to manage. 

One such separation would be game logic and UI. Rather than directly coupling the player's health to the UI representation of the player's health, it makes sense to have them communicate via some type of messaging system. An event seems like a natural fit.

```cs
public class PlayerSpawnedArgs : EventArgs {
    public readonly Player Player;
}

public class HealthChangedArgs : EventArgs {
    public readonly float PreviousValue;
    public readonly float CurrentValue;
}

class Healthbar : UIElement {
    private float _currentHealth;
    // use getter/setter to automatically redraw
    // the UI on state changes
    private float currentHealth {
        get => _currentHealth;
        set {
            if (value != _currentHealth) {
                _currentHealth = value;
                Redraw();
            }
        }
    }

    protected void OnInitialize() {
        // register to the player spawn event
        PlayerSpawnEvent.Register(OnPlayerSpawned);
    }

    private void OnPlayerSpawned(object sender, PlayerSpawnedArgs args) {
        // subscribe specifically to that player's events
        args.Player.health.Register(OnHealthChanged);
    }

    private void OnHealthChanged(object sender, HealthChangedArgs args) {
        currentHealth = args.CurrentValue;
    }
}
```

This will work for many cases but it's unfortunate it needs to manage subscriptions to two events in order to get the updates it needs. If we ever needed to add more than one player that renders a healthbar, this solution would no longer be sufficient. We could circumvent this by making health updates dispatched globally but that wold come at the cost of checking *every* health update just to find a player. This solution would be much more appealing, however, if every entity with health had a healthbar. One happy medium would be to create a more specific global event for player health updates. `Healthbar` can then simply register to that event without needing to concern itself with the particulars of spawning.

With all this event-driven programming comes another downside: events are opaque and difficult to debug. When an architecture relies heavily on events, it becomes increasingly important to have outstanding documentation. It's an unfortunate sacrifice to be made in exchange for the highly decoupled nature of events. Some game engines have made attempts to offer better insight into event connections but it's often only a mild remedy.

In order to better query the state of an event-driven application, [Martin Fowler's Event Sourcing post](https://martinfowler.com/eaaDev/EventSourcing.html) provides some additional respite. It's worth reading if you're working with many events in your game. Essentially, tracking the source of an event *in* the newly raised event allows subscribers to walk the chain of source events in reverse to determine additional context.

## Enter ECS
These were just a few pain points of game architecture that I've come across when making games. I've tried different solutions each time to varying levels of success. While I don't think there exists a one-size-fits-all solution to every architectural decision in video games, I *do* believe that reframing how we think about our architectural goals can make some problems diminish or even disappear. This is especially helpful if the affected problems were persistent across multiple projects.

Entity-Component-System (ECS) is a data-oriented approach that has resolved many of the above issues for me. It comes with its own architectural challenges, especially since the pattern has been rapidly evolving due to its recent explosion of popularity.

There are many guides attempting to explain ECS in a simple terms. This can be a bit challenging since the approach may run counter to the fundamental understanding of game architecture for many new readers. Additionally, there are different types and implementations of ECS which sometimes pollute the overall message. [Sander Mertens](https://ajmmertens.medium.com/), the author of [FLECS](https://github.com/SanderMertens/flecs), has contributed a substantial amount to the development and education of ECS. Their [FAQ](https://github.com/SanderMertens/ecs-faq) is a valuable resource to have. I'm going to try to provide a high level explanation of ECS but I recommend looking to other resources if this doesn't make sense.

### Entities

An entity is an identifier for an object. It has no data, properties, or behaviors by itself. It is simply a marker for something that exists. In some implementations of ECS, this could be as simple as an unsigned integer. These identifiers are the primary way of fetching *Component* data.

### Components

Components hold data and belong to an entity. Some implementations have limitations on what kind of data can be housed but conceptually it can be anything. A player's level, their position in the world, their current health, and the current input state of a gamepad would all be stored in a component.

Components are often stored contiguously in memory (such as in an array). The entity is used to fetch data from that container.


### An example
Let's pause for a moment to consider the relationship between entities and components. If we were to focus purely on simplicity, we could implement these concepts in the following way.


```cs
// this is our component definition
struct Health {
    public double Current;
    public double Max;
}

Health[] health = new Health[100]; // this is our component container

int player = 0; // this is our entity
double currentHealth = health[player].current; // accessing our component data
Console.WriteLine($"The player's current health is {currentHealth}");
```


I would like to restate that the actual interface will depend on the  library and the decisions its developers have made. Different implementations have different tradeoffs and limitations that may change the internal representation of an entity or component.

If we extend our example just a little bit to include multiple components, we would end up with multiple containers (arrays) too. This has an interesting implication in that it allows us to visualize our data in a more intuitive way: a table.

| Entity | Name     | Health.Current | Health.Max |
| ------ | -------- | -------------- | ---------- |
| 0      | "Kain"   | 100            | 100        |
| 1      | "Raziel" | 5              | 75         |
| 2      | "Janos"  | 0              | 1000       |

Our entity is the row ID and each successive column is its associated component data. In cases where an entity does not have a component, we can think of its value as `NULL`.

| Entity | Name     | Health.Current | Health.Max | Weapon.Name   |
| ------ | -------- | -------------- | ---------- | ------------- |
| 0      | "Kain"   | 100            | 100        | "Soul Reaver" |
| 1      | "Raziel" | 5              | 75         | "Soul Reaver" |
| 2      | "Janos"  | 0              | 1000       | `NULL`        |


### Systems

A system represents a behavior. This is where game and application logic lives. Systems receive a list of entities and iterate over that list to perform work on their component data. They may also create and destroy entities or attach and remove components. If we want to write a system which applies movement to an entity, we could check for the existence of a `Position` and `Velocity` components.

```cs
foreach(var entity of entities) {
    if (entity.hasComponent(Velocity) && entity.hasComponent(Position)) {
        
    }
}
```

## Going further (Conditional components, Queries)
Most ECS systems can query the state to refine the list of entities they receive to what is most pertinent. Some have scheduling and ordering systems for systems. 

## Performance
its fast

## Issues with ECS
Mental modeling is hard. Complex queries are hard. ECS is hard.

## A Solution?
something something game state as a database, expressive querying
https://ajmmertens.medium.com/why-it-is-time-to-start-thinking-of-games-as-databases-e7971da33ac3

## Going forward
continued evolution of ecs which can broadly respond to specific use cases and needs

### What I'm looking for
* constructive feedback and discussion about the general direction i'm putting forward
* potential implementations and data structures that allow ecs to perform as outlined
* potential pitfalls that ecs may not be able to answer

### Not on topic but will still accept
* improvements and specific optimizations to my prototype
* constructive feedback on the specific contents of this post (eg. code examples)

### What I'm not looking for
* criticism and discussion about my specific prototype implementation
* paradigm, language, and coding style debates
    * i'm not interested in debating the merits of one paradigm or language versus another
    * i'm extremely not interested in K&R versus Allman indentation style arguments. [yes i am a monster](https://youtu.be/raQ3iHhE_Kk?t=1093) (but that's not a secret)
    * as stated in the preface, all of these are approaches that have worked best for me and how i think. if OO or whatever else works for you, you should use that!
* criticism about the code examples in this post 
    * how best to solve a problem via OOP is not the point of this post
    * any feedback on how to better demonstrate solutions to the example problems are ok though. i want to fairly represent the alternatives


{% endblock article %}
